# -*- coding: utf-8 -*-
from uuid import uuid4
import traceback
import sys
from flask import make_response, request
from openeo_grass_gis_driver.process_graph_db import GraphDB
from openeo_grass_gis_driver.actinia_processing.config import \
     Config as ActiniaConfig
from openeo_grass_gis_driver.actinia_processing.actinia_interface import \
     ActiniaInterface
from openeo_grass_gis_driver.authentication import ResourceBase
from openeo_grass_gis_driver.models.process_graph_schemas import \
     ProcessGraphListEntry, ProcessGraphList
from openeo_grass_gis_driver.models.error_schemas import ErrorSchema

__license__ = "Apache License, Version 2.0"
__author__ = "Sören Gebbert"
__copyright__ = "Copyright 2018, Sören Gebbert, mundialis"
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"


class ProcessGraphs(ResourceBase):
    """The /process_graphs endpoint implementation"""

    def __init__(self):
        ResourceBase.__init__(self)
        self.iface = ActiniaInterface()
        self.iface.set_auth(ActiniaConfig.USER, ActiniaConfig.PASSWORD)
        self.graph_db = GraphDB()

    def get(self):
        """Return all jobs in the job database"""
        # TODO: Implement user specific database access

        process_graphs = []

        for key in self.graph_db:
            graph = self.graph_db[key]

            title = None
            if "title" in graph:
                title = graph["title"]
            description = None
            if "description" in graph:
                description = graph["description"]
            entry = ProcessGraphListEntry(
                title=title, description=description, id=key)

            process_graphs.append(entry)

        return ProcessGraphList(
            process_graphs=process_graphs).as_response(
            http_status=200)

    # no longer supported, replaced by ProcessGraphId
    def post(self):
        try:
            """Store a process graph in the graph database"""
            # TODO: Implement user specific database access

            process_graph_id = f"user-graph-{str(uuid4())}"

            process_graph = request.get_json()
            self.graph_db[process_graph_id] = process_graph

            return make_response(process_graph_id, 201)
        except Exception:

            e_type, e_value, e_tb = sys.exc_info()
            traceback_model = dict(message=str(e_value),
                                   traceback=traceback.format_tb(e_tb),
                                   type=str(e_type))
            return ErrorSchema(
                id="1234567890",
                code=2,
                message=str(traceback_model)).as_response(
                http_status=400)

    def delete(self):
        """Clear the process graph database"""
        self.graph_db.clear()
        return make_response(
            "All process graphs have been successfully deleted", 204)
